# -*- coding: utf-8 -*-

from __future__ import (
    print_function,
    unicode_literals,
    absolute_import,
    division)

import hashlib
import hmac
import six


try:
    compare_digest = hmac.compare_digest

except AttributeError:
    # Not secure when the length is supposed to be kept secret
    def compare_digest(a, b):
        if len(a) != len(b):
            return False

        return reduce(
            lambda x, y: x | y, [ord(x) ^ ord(y) for x, y in zip(a, b)]) == 0


def sign(secret, string_to_sign):
    return six.text_type(
        hmac.new(
            secret.encode('utf8'),
            string_to_sign.encode('utf8'),
            hashlib.sha256).hexdigest())


def verify(secret, string_to_sign, signature):
    return compare_digest(signature, sign(secret, string_to_sign))



# -*- coding: utf-8 -*-

from __future__ import (
    print_function,
    unicode_literals,
    absolute_import,
    division)

import collections
import hashlib
import os
import re
import six
import time

from pusher.util import (
    ensure_text,
    pusher_url_re,
    doc_string)

from pusher.pusher_client import PusherClient
from pusher.authentication_client import AuthenticationClient


class Pusher(object):
    """Client for the Pusher HTTP API.

    This client supports various backend adapters to support various http
    libraries available in the python ecosystem.

    :param app_id:  a pusher application identifier
    :param key:     a pusher application key
    :param secret:  a pusher application secret token
    :param ssl:     Whenever to use SSL or plain HTTP
    :param host:    Used for custom host destination
    :param port:    Used for custom port destination
    :param timeout: Request timeout (in seconds)
    :param encryption_master_key: deprecated, use encryption_master_key_base64
    :param encryption_master_key_base64: Used to derive a shared secret
      between server and the clients for payload encryption/decryption
    :param cluster: Convention for clusters other than the original Pusher cluster.
      Eg: 'eu' will resolve to the api-eu.pusherapp.com host
    :param backend: an http adapter class (AsyncIOBackend, RequestsBackend,
      SynchronousBackend, TornadoBackend)
    :param backend_options: additional backend
    """
    def __init__(
            self,
            app_id,
            key,
            secret,
            ssl=True,
            host=None,
            port=None,
            timeout=5,
            cluster=None,
            encryption_master_key=None,
            encryption_master_key_base64=None,
            json_encoder=None,
            json_decoder=None,
            backend=None,
            **backend_options):

        self._pusher_client = PusherClient(
            app_id,
            key,
            secret,
            ssl,
            host,
            port,
            timeout,
            cluster,
            encryption_master_key,
            encryption_master_key_base64,
            json_encoder,
            json_decoder,
            backend,
            **backend_options)

        self._authentication_client = AuthenticationClient(
            app_id,
            key,
            secret,
            ssl,
            host,
            port,
            timeout,
            cluster,
            encryption_master_key,
            encryption_master_key_base64,
            json_encoder,
            json_decoder,
            backend,
            **backend_options)


    @classmethod
    def from_url(cls, url, **options):
        """Alternative constructor that extracts the information from a URL.

        :param url: String containing a URL

        Usage::

          >> from pusher import Pusher
          >> p =
            Pusher.from_url("http://mykey:mysecret@api.pusher.com/apps/432")
        """
        m = pusher_url_re.match(ensure_text(url, "url"))
        if not m:
            raise Exception("Unparsable url: %s" % url)

        ssl = m.group(1) == 'https'

        options_ = {
            'key': m.group(2),
            'secret': m.group(3),
            'host': m.group(4),
            'app_id': m.group(5),
            'ssl': ssl}

        options_.update(options)

        return cls(**options_)


    @classmethod
    def from_env(cls, env='PUSHER_URL', **options):
        """Alternative constructor that extracts the information from an URL
        stored in an environment variable. The pusher heroku addon will set
        the PUSHER_URL automatically when installed for example.

        :param env: Name of the environment variable

        Usage::

          >> from pusher import Pusher
          >> c = Pusher.from_env("PUSHER_URL")
        """
        val = os.environ.get(env)
        if not val:
            raise Exception("Environment variable %s not found" % env)

        return cls.from_url(val, **options)


    @doc_string(PusherClient.trigger.__doc__)
    def trigger(self, channels, event_name, data, socket_id=None):
        return self._pusher_client.trigger(
            channels, event_name, data, socket_id)


    @doc_string(PusherClient.trigger_batch.__doc__)
    def trigger_batch(self, batch=[], already_encoded=False):
        return self._pusher_client.trigger_batch(batch, already_encoded)

    @doc_string(PusherClient.channels_info.__doc__)
    def channels_info(self, prefix_filter=None, attributes=[]):
        return self._pusher_client.channels_info(prefix_filter, attributes)


    @doc_string(PusherClient.channel_info.__doc__)
    def channel_info(self, channel, attributes=[]):
        return self._pusher_client.channel_info(channel, attributes)

    @doc_string(PusherClient.users_info.__doc__)
    def users_info(self, channel):
        return self._pusher_client.users_info(channel)

    @doc_string(PusherClient.terminate_user_connections.__doc__)
    def terminate_user_connections(self, user_id):
        return self._pusher_client.terminate_user_connections(user_id)

    @doc_string(AuthenticationClient.authenticate.__doc__)
    def authenticate(self, channel, socket_id, custom_data=None):
        return self._authentication_client.authenticate(
            channel, socket_id, custom_data)


    @doc_string(AuthenticationClient.validate_webhook.__doc__)
    def validate_webhook(self, key, signature, body):
        return self._authentication_client.validate_webhook(
            key, signature, body)



# -*- coding: utf-8 -*-

from __future__ import (
    print_function,
    unicode_literals,
    absolute_import,
    division)


class PusherError(Exception):
    pass


class PusherBadRequest(PusherError):
    pass


class PusherBadAuth(PusherError):
    pass


class PusherForbidden(PusherError):
    pass


class PusherBadStatus(PusherError):
    pass



# -*- coding: utf-8 -*-

from __future__ import (
    print_function,
    unicode_literals,
    absolute_import,
    division)

import collections
import hashlib
import json
import os
import re
import six
import time
import base64

from pusher.util import (
    ensure_text,
    ensure_binary,
    validate_channel,
    validate_socket_id,
    channel_name_re
    )

from pusher.client import Client
from pusher.http import GET, POST, Request, request_method
from pusher.signature import sign, verify
from pusher.crypto import *


class AuthenticationClient(Client):
    def __init__(
            self,
            app_id,
            key,
            secret,
            ssl=True,
            host=None,
            port=None,
            timeout=5,
            cluster=None,
            encryption_master_key=None,
            encryption_master_key_base64=None,
            json_encoder=None,
            json_decoder=None,
            backend=None,
            **backend_options):

        super(AuthenticationClient, self).__init__(
            app_id,
            key,
            secret,
            ssl,
            host,
            port,
            timeout,
            cluster,
            encryption_master_key,
            encryption_master_key_base64,
            json_encoder,
            json_decoder,
            backend,
            **backend_options)


    def authenticate(self, channel, socket_id, custom_data=None):
        """Used to generate delegated client subscription token.

        :param channel: name of the channel to authorize subscription to
        :param socket_id: id of the socket that requires authorization
        :param custom_data: used on presence channels to provide user info
        """
        channel = validate_channel(channel)

        if not channel_name_re.match(channel):
            raise ValueError('Channel should be a valid channel, got: %s' % channel)

        socket_id = validate_socket_id(socket_id)

        if custom_data:
            custom_data = json.dumps(custom_data, cls=self._json_encoder)

        string_to_sign = "%s:%s" % (socket_id, channel)

        if custom_data:
            string_to_sign += ":%s" % custom_data

        signature = sign(self.secret, string_to_sign)

        auth = "%s:%s" % (self.key, signature)
        response_payload = { "auth": auth }

        if is_encrypted_channel(channel):
            shared_secret = generate_shared_secret(
                ensure_binary(channel, "channel"), self._encryption_master_key)
            shared_secret_b64 = base64.b64encode(shared_secret)
            response_payload["shared_secret"] = shared_secret_b64

        if custom_data:
            response_payload['channel_data'] = custom_data

        return response_payload


    def validate_webhook(self, key, signature, body):
        """Used to validate incoming webhook messages. When used it guarantees
        that the sender is Pusher and not someone else impersonating it.

        :param key: key used to sign the body
        :param signature: signature that was given with the body
        :param body: content that needs to be verified
        """
        key = ensure_text(key, "key")
        signature = ensure_text(signature, "signature")
        body = ensure_text(body, "body")

        if key != self.key:
            return None

        if not verify(self.secret, body, signature):
            return None

        try:
            body_data = json.loads(body, cls=self._json_decoder)

        except ValueError:
            return None

        time_ms = body_data.get('time_ms')
        if not time_ms:
            return None

        if abs(time.time()*1000 - time_ms) > 300000:
            return None

        return body_data



# -*- coding: utf-8 -*-

from __future__ import (
    print_function,
    unicode_literals,
    absolute_import,
    division)

import aiohttp
import asyncio

from pusher.http import process_response


class AsyncIOBackend:
    def __init__(self, client):
        """Adapter for the requests module.

        :param client:  pusher.Client object
        """
        self.client = client


    @asyncio.coroutine
    def send_request(self, request):
        session = response = None
        try:
            session = aiohttp.ClientSession()
            response = yield from session.request(
                request.method,
                "%s%s" % (request.base_url, request.path),
                params=request.query_params,
                data=request.body,
                headers=request.headers,
                timeout=self.client.timeout
            )
            body = yield from response.text('utf-8')
            return process_response(response.status, body)
        finally:
            if response is not None:
                response.close()
            if session is not None:
                yield from session.close()



# -*- coding: utf-8 -*-

from __future__ import (
    print_function,
    unicode_literals,
    absolute_import,
    division)

import json
import re
import six
import sys
import base64

channel_name_re = re.compile(r'\A[-a-zA-Z0-9_=@,.;]+\Z')
app_id_re = re.compile(r'\A[0-9]+\Z')
pusher_url_re = re.compile(r'\A(http|https)://(.*):(.*)@(.*)/apps/([0-9]+)\Z')
socket_id_re = re.compile(r'\A\d+\.\d+\Z')


if sys.version_info < (3,):
    text = 'a unicode string'
else:
    text = 'a string'


if sys.version_info < (3,):
    byte_type = 'a python2 str'
else:
    byte_type = 'a python3 bytes'

def ensure_text(obj, name):
    if isinstance(obj, six.text_type):
        return obj

    if isinstance(obj, six.string_types):
       return six.text_type(obj)

    if isinstance(obj, six.binary_type):
      return bytes(obj).decode('utf-8')

    raise TypeError("%s should be %s instead it is a %s" % (name, text, type(obj)))

def ensure_binary(obj, name):
    """
    ensure_binary() ensures that the value is a
    python2 str or python3 bytes
    more on this here: https://pythonhosted.org/six/#six.binary_type
    """
    if isinstance(obj, six.binary_type):
      return obj

    if isinstance(obj, six.text_type) or isinstance(obj, six.string_types):
       return obj.encode("utf-8")

    raise TypeError("%s should be %s instead it is a %s" % (name, byte_type, type(obj)))


def is_base64(s):
    """
    is_base64 tests whether a string is valid base64 by testing that it round-trips accurately.
    This is required because python 2.7 does not have a Validate option to the decoder.
    """
    try:
        s = six.ensure_binary(s, "utf-8")
        return base64.b64encode(base64.b64decode(s)) == s
    except Exception as e:
        return False

def validate_user_id(user_id):
    user_id = ensure_text(user_id, "user_id")

    length = len(user_id)
    if length == 0:
        raise ValueError("User id is empty")

    if length > 200:
        raise ValueError("User id too long: '{}'".format(user_id))

    if not channel_name_re.match(user_id):
        raise ValueError("Invalid user id: '{}'".format(user_id))

    return user_id

def validate_channel(channel):
    channel = ensure_text(channel, "channel")

    if len(channel) > 200:
        raise ValueError("Channel too long: %s" % channel)

    if not channel_name_re.match(channel):
        raise ValueError("Invalid Channel: %s" % channel)

    return channel


def validate_socket_id(socket_id):
    socket_id = ensure_text(socket_id, "socket_id")

    if not socket_id_re.match(socket_id):
        raise ValueError("Invalid socket ID: %s" % socket_id)

    return socket_id


def join_attributes(attributes):
    return six.text_type(',').join(attributes)


def data_to_string(data, json_encoder):
    if isinstance(data, six.string_types):
        return ensure_text(data, "data")

    else:
        return json.dumps(data, cls=json_encoder)


def doc_string(doc):
    def decorator(f):
        f.__doc__ = doc
        return f

    return decorator



# -*- coding: utf-8 -*-

from __future__ import (
    print_function,
    unicode_literals,
    absolute_import,
    division)

import six

from pusher.util import ensure_text, ensure_binary, app_id_re
from pusher.crypto import parse_master_key


class Client(object):
    def __init__(
            self,
            app_id,
            key,
            secret,
            ssl=True,
            host=None,
            port=None,
            timeout=5,
            cluster=None,
            encryption_master_key=None,
            encryption_master_key_base64=None,
            json_encoder=None,
            json_decoder=None,
            backend=None,
            **backend_options):

        if backend is None:
              from .requests import RequestsBackend
              backend = RequestsBackend

        self._app_id = ensure_text(app_id, "app_id")
        if not app_id_re.match(self._app_id):
              raise ValueError("Invalid app id")

        self._key = ensure_text(key, "key")
        self._secret = ensure_text(secret, "secret")

        if not isinstance(ssl, bool):
              raise TypeError("SSL should be a boolean")

        self._ssl = ssl

        if host:
            self._host = ensure_text(host, "host")
        elif cluster:
            self._host = (
                six.text_type("api-%s.pusher.com") %
                ensure_text(cluster, "cluster"))
        else:
            self._host = six.text_type("api.pusherapp.com")

        if port and not isinstance(port, six.integer_types):
              raise TypeError("port should be an integer")

        self._port = port or (443 if ssl else 80)

        if not isinstance(timeout, six.integer_types):
              raise TypeError("timeout should be an integer")

        self._timeout = timeout
        self._json_encoder = json_encoder
        self._json_decoder = json_decoder

        self._encryption_master_key = parse_master_key(encryption_master_key, encryption_master_key_base64)

        self.http = backend(self, **backend_options)


    @property
    def app_id(self):
        return self._app_id

    @property
    def key(self):
        return self._key

    @property
    def secret(self):
        return self._secret

    @property
    def host(self):
        return self._host

    @property
    def port(self):
        return self._port

    @property
    def timeout(self):
        return self._timeout

    @property
    def ssl(self):
        return self._ssl

    @property
    def scheme(self):
        return 'https' if self.ssl else 'http'



# -*- coding: utf-8 -*-

from __future__ import (
    print_function,
    unicode_literals,
    absolute_import,
    division)

import copy
import hashlib
import json
import six
import time

from pusher.util import doc_string
from pusher.errors import *
from pusher.signature import sign
from pusher.version import VERSION


GET, POST, PUT, DELETE = "GET", "POST", "PUT", "DELETE"


class RequestMethod(object):
    def __init__(self, client, f):
        self.client = client
        self.f = f


    def __call__(self, *args, **kwargs):
        return self.client.http.send_request(self.make_request(*args, **kwargs))


    def make_request(self, *args, **kwargs):
        return self.f(self.client, *args, **kwargs)


def request_method(f):
    @property
    @doc_string(f.__doc__)
    def wrapped(self):
        return RequestMethod(self, f)

    return wrapped


def make_query_string(params):
    return '&'.join(map('='.join, sorted(params.items(), key=lambda x: x[0])))


def process_response(status, body):
    if status == 200 or status == 202:
        return json.loads(body)

    elif status == 400:
        raise PusherBadRequest(body)

    elif status == 401:
        raise PusherBadAuth(body)

    elif status == 403:
        raise PusherForbidden(body)

    else:
        raise PusherBadStatus("%s: %s" % (status, body))


class Request(object):
    """Represents the request to be made to the Pusher API.

    An instance of that object is passed to the backend's send_request method
    for each request.

    :param client: an instance of pusher.Client
    :param method: HTTP method as a string
    :param path: The target path on the destination host
    :param params: Query params or body depending on the method
    """
    def __init__(self, client, method, path, params=None):
        if params is None:
            params = {}

        self.client = client
        self.method = method
        self.path = path
        self.params = copy.copy(params)
        if method == POST:
            self.body = six.text_type(json.dumps(params)).encode('utf8')
            self.query_params = {}

        elif method == GET:
            self.body = bytes()
            self.query_params = params

        else:
            raise NotImplementedError("Only GET and POST supported")

        self._generate_auth()


    def _generate_auth(self):
        self.body_md5 = hashlib.md5(self.body).hexdigest()
        self.query_params.update({
            'auth_key': self.client.key,
            'body_md5': six.text_type(self.body_md5),
            'auth_version': '1.0',
            'auth_timestamp': '%.0f' % time.time()})

        auth_string = '\n'.join([
            self.method,
            self.path,
            make_query_string(self.query_params)])

        self.query_params['auth_signature'] = sign(
            self.client.secret, auth_string)


    @property
    def query_string(self):
        return make_query_string(self.query_params)


    @property
    def signed_path(self):
        return "%s?%s" % (self.path, self.query_string)


    @property
    def url(self):
        return "%s%s" % (self.base_url, self.signed_path)


    @property
    def base_url(self):
        return (
            "%s://%s:%s" %
            (self.client.scheme, self.client.host, self.client.port))


    @property
    def headers(self):
        hdrs = {"X-Pusher-Library": "pusher-http-python " + VERSION}
        if self.method == POST:
            hdrs["Content-Type"] = "application/json"

        return hdrs



# -*- coding: utf-8 -*-

from __future__ import (
    print_function,
    unicode_literals,
    absolute_import,
    division)

import six
import tornado
import tornado.httpclient

from tornado.concurrent import Future

from pusher.http import process_response


class TornadoBackend(object):
    """Adapter for the tornado.httpclient module.

    :param client:   pusher.Client object
    :param options:  options for the httpclient.HTTPClient constructor
    """
    def __init__(self, client, **options):
        self.client = client
        self.options = options
        self.http = tornado.httpclient.AsyncHTTPClient()


    def send_request(self, request):
        method = request.method
        data = request.body
        headers = {'Content-Type': 'application/json'}
        future = Future()

        def process_response_future(response):
            if response.exception() is not None:
                future.set_exception(response.exception())

            else:
                result = response.result()
                code = result.code
                body = (result.body or b'').decode('utf8')
                future.set_result(process_response(code, body))

        request = tornado.httpclient.HTTPRequest(
            request.url,
            method=method,
            body=data,
            headers=headers,
            request_timeout=self.client.timeout,
            **self.options)

        response_future = self.http.fetch(request, raise_error=False)
        response_future.add_done_callback(process_response_future)

        return future



# -*- coding: utf-8 -*-

from __future__ import (
    print_function,
    unicode_literals,
    absolute_import,
    division)

from pusher.http import process_response

import requests
import sys
import os


if sys.version_info < (3,):
    import urllib3.contrib.pyopenssl
    urllib3.contrib.pyopenssl.inject_into_urllib3()

CERT_PATH = os.path.dirname(os.path.abspath(__file__)) + '/cacert.pem'


class RequestsBackend(object):
    """Adapter for the requests module.

    :param client:  pusher.Client object
    :param options: key-value passed into the requests.request constructor
    """
    def __init__(self, client, **options):
        self.client = client
        self.options = options
        if self.client.ssl:
            self.options.update({'verify': CERT_PATH})
        self.session = requests.Session()


    def send_request(self, request):
        resp = self.session.request(
            request.method,
            request.url,
            headers=request.headers,
            data=request.body,
            timeout=self.client.timeout,
            **self.options)

        return process_response(resp.status_code, resp.text)



# -*- coding: utf-8 -*-

from __future__ import (
    print_function,
    unicode_literals,
    absolute_import,
    division)

import sys
# Abstract Base Classes were moved into collections.abc in Python 3.3
if sys.version_info >= (3,3):
    import collections.abc as collections
else:
    import collections
import hashlib
import os
import re
import six
import time
import json
import string

from pusher.util import (
    ensure_text,
    validate_channel,
    validate_socket_id,
    validate_user_id,
    join_attributes,
    data_to_string)

from pusher.client import Client
from pusher.http import GET, POST, Request, request_method
from pusher.crypto import *
import random
from datetime import datetime


class PusherClient(Client):
    def __init__(
            self,
            app_id,
            key,
            secret,
            ssl=True,
            host=None,
            port=None,
            timeout=5,
            cluster=None,
            encryption_master_key=None,
            encryption_master_key_base64=None,
            json_encoder=None,
            json_decoder=None,
            backend=None,
            **backend_options):

        super(PusherClient, self).__init__(
                app_id,
                key,
                secret,
                ssl,
                host,
                port,
                timeout,
                cluster,
                encryption_master_key,
                encryption_master_key_base64,
                json_encoder,
                json_decoder,
                backend,
                **backend_options)


    @request_method
    def trigger(self, channels, event_name, data, socket_id=None):
        """Trigger an event on one or more channels, see:

        http://pusher.com/docs/rest_api#method-post-event
        """
        if isinstance(channels, six.string_types):
            channels = [channels]

        if isinstance(channels, dict) or not isinstance(
                channels, (collections.Sized, collections.Iterable)):
            raise TypeError("Expected a single or a list of channels")

        if len(channels) > 100:
            raise ValueError("Too many channels")

        event_name = ensure_text(event_name, "event_name")
        if len(event_name) > 200:
            raise ValueError("event_name too long")

        data = data_to_string(data, self._json_encoder)
        if sys.getsizeof(data) > 10240:
            raise ValueError("Too much data")

        channels = list(map(validate_channel, channels))

        if len(channels) > 1:
            for chan in channels:
                if is_encrypted_channel(chan):
                    raise ValueError("You cannot trigger to multiple channels when using encrypted channels")

        if is_encrypted_channel(channels[0]):
            data = json.dumps(encrypt(channels[0], data, self._encryption_master_key), ensure_ascii=False)

        params = {
            'name': event_name,
            'channels': channels,
            'data': data}

        if socket_id:
            params['socket_id'] = validate_socket_id(socket_id)

        return Request(self, POST, "/apps/%s/events" % self.app_id, params)

    @request_method
    def trigger_batch(self, batch=[], already_encoded=False):
        """Trigger multiple events with a single HTTP call.

        http://pusher.com/docs/rest_api#method-post-batch-events
        """
        if not already_encoded:
            for event in batch:
                validate_channel(event['channel'])

                event_name = ensure_text(event['name'], "event_name")
                if len(event['name']) > 200:
                    raise ValueError("event_name too long")

                event['data'] = data_to_string(event['data'], self._json_encoder)

                if sys.getsizeof(event['data']) > 10240:
                    raise ValueError("Too much data")

                if is_encrypted_channel(event['channel']):
                    event['data'] = json.dumps(encrypt(event['channel'], event['data'], self._encryption_master_key), ensure_ascii=False)

        params = {
            'batch': batch}

        return Request(
            self, POST, "/apps/%s/batch_events" % self.app_id, params)

    @request_method
    def channels_info(self, prefix_filter=None, attributes=[]):
        """Get information on multiple channels, see:

        http://pusher.com/docs/rest_api#method-get-channels
        """
        params = {}
        if attributes:
            params['info'] = join_attributes(attributes)

        if prefix_filter:
            params['filter_by_prefix'] = ensure_text(
                prefix_filter, "prefix_filter")

        return Request(
            self, GET, six.text_type("/apps/%s/channels") % self.app_id, params)


    @request_method
    def channel_info(self, channel, attributes=[]):
        """Get information on a specific channel, see:

        http://pusher.com/docs/rest_api#method-get-channel
        """
        validate_channel(channel)

        params = {}
        if attributes:
            params['info'] = join_attributes(attributes)

        return Request(
            self, GET, "/apps/%s/channels/%s" % (self.app_id, channel), params)


    @request_method
    def users_info(self, channel):
        """Fetch user ids currently subscribed to a presence channel

        http://pusher.com/docs/rest_api#method-get-users
        """
        validate_channel(channel)

        return Request(
            self, GET, "/apps/%s/channels/%s/users" % (self.app_id, channel))

    @request_method
    def terminate_user_connections(self, user_id):
        validate_user_id(user_id)
        return Request(
            self, POST, "/users/{}/terminate_connections".format(user_id), {})



# -*- coding: utf-8 -*-
from setuptools import setup
import os
import re

# Lovingly adapted from https://github.com/kennethreitz/requests/blob/39d693548892057adad703fda630f925e61ee557/setup.py#L50-L55
with open(os.path.join(os.path.dirname(os.path.abspath(__file__)), 'pusher/version.py'), 'r') as fd:
    VERSION = re.search(r'^VERSION = [\']([^\']*)[\']',
                        fd.read(), re.MULTILINE).group(1)

if not VERSION:
    raise RuntimeError('Ensure `VERSION` is correctly set in ./pusher/version.py')

setup(
    name='pusher',
    version=VERSION,
    description='A Python library to interract with the Pusher Channels API',
    long_description='A Python library to interract with the Pusher Channels API',
    long_description_type='text/x-rst',
    url='https://github.com/pusher/pusher-http-python',
    author='Pusher',
    author_email='support@pusher.com',
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Topic :: Internet :: WWW/HTTP',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 3',
    ],
    keywords='pusher rest realtime websockets service',
    license='MIT',

    packages=[
        'pusher'
    ],

    install_requires=[
        'six',
        'requests>=2.3.0',
        'urllib3',
        'pyopenssl',
        'ndg-httpsclient',
        'pyasn1',
        'pynacl'
    ],

    tests_require=['nose', 'mock', 'HTTPretty'],

    extras_require={
        'aiohttp': ['aiohttp>=0.20.0'],
        'tornado': ['tornado>=5.0.0']
    },

    package_data={
        'pusher': ['cacert.pem']
    },

    test_suite='pusher_tests',
)



# -*- coding: utf-8 -*-

from __future__ import (
    print_function,
    unicode_literals,
    absolute_import,
    division)

import hashlib
import nacl
import base64
import binascii
import warnings

from pusher.util import (
    ensure_text,
    ensure_binary,
    data_to_string,
    is_base64)

import nacl.secret
import nacl.utils

# The prefix any e2e channel must have
ENCRYPTED_PREFIX = 'private-encrypted-'

def is_encrypted_channel(channel):
    """
    is_encrypted_channel() checks if the channel is encrypted by verifying the prefix
    """
    if channel.startswith(ENCRYPTED_PREFIX):
        return True
    return False

def parse_master_key(encryption_master_key, encryption_master_key_base64):
    """
    parse_master_key validates, parses and returns the bytes of the encryption master key
    from the constructor arguments.
    At present there is a deprecated "raw" key and a suggested base64 encoding.
    """
    if encryption_master_key is not None and encryption_master_key_base64 is not None:
        raise ValueError("Do not provide both encryption_master_key and encryption_master_key_base64. " +
                "encryption_master_key is deprecated, provide only encryption_master_key_base64")

    if encryption_master_key is not None:
        warnings.warn("`encryption_master_key` is deprecated, please use `encryption_master_key_base64`")
        if len(encryption_master_key) == 32:
            return ensure_binary(encryption_master_key, "encryption_master_key")
        else:
            raise ValueError("encryption_master_key must be 32 bytes long")

    if encryption_master_key_base64 is not None:
        if is_base64(encryption_master_key_base64):
            decoded = base64.b64decode(encryption_master_key_base64)

            if len(decoded) == 32:
                return decoded
            else:
                raise ValueError("encryption_master_key_base64 must be a base64 string which decodes to 32 bytes")
        else:
            raise ValueError("encryption_master_key_base64 must be valid base64")

    return None

def generate_shared_secret(channel, encryption_master_key):
    """
    generate_shared_secret() takes a six.binary_type (python2 str or python3 bytes) channel name and encryption_master_key
    and returns the sha256 hash in six.binary_type format
    """
    if encryption_master_key is None:
        raise ValueError("No master key was provided for use with encrypted channels. Please provide encryption_master_key_base64 as an argument to the Pusher SDK")

    hashable = channel + encryption_master_key
    return hashlib.sha256(hashable).digest()

def encrypt(channel, data, encryption_master_key, nonce=None):
    """
    encrypt() encrypts the provided payload specified in the 'data' parameter
    """
    channel = ensure_binary(channel, "channel")
    shared_secret = generate_shared_secret(channel, encryption_master_key)
    # the box setup to seal/unseal data payload
    box = nacl.secret.SecretBox(shared_secret)

    if nonce is None:
        nonce = nacl.utils.random(nacl.secret.SecretBox.NONCE_SIZE)
    else:
        nonce = ensure_binary(nonce, "nonce")

    # convert nonce to base64
    nonce_b64 = base64.b64encode(nonce)

    # encrypt the data payload with nacl
    encrypted = box.encrypt(data.encode("utf-8"), nonce)

    # obtain the ciphertext
    cipher_text = encrypted.ciphertext
    # encode cipertext to base64
    cipher_text_b64 = base64.b64encode(cipher_text)

    # format output
    return { "nonce" : nonce_b64.decode("utf-8"), "ciphertext": cipher_text_b64.decode("utf-8") }



# -*- coding: utf-8 -*-

from __future__ import (
    print_function,
    unicode_literals,
    absolute_import,
    division)

from google.appengine.api import urlfetch

from pusher.http import process_response


class GAEBackend(object):
    """
    Adapter for the URLFetch Module. Necessary for using this library with
    Google App Engine
    """
    def __init__(self, client, **options):
        self.client = client
        self.options = options


    def send_request(self, request):
        resp = urlfetch.fetch(
            url=request.url,
            headers=request.headers,
            method=request.method,
            payload=request.body,
            deadline=self.client.timeout,
            **self.options)

        return process_response(resp.status_code, resp.content)
